﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ContainerShip
{
    public partial class FormMapWithSetShip : Form
    {
       
        private MapWithSetShipGeneric<DrawingObjectShip, AbstractMap> _mapShipCollectionGeneric;
        
       
        public FormMapWithSetShip()
        {
            InitializeComponent();
            
        }
        
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    map = new SimpleMap();
                    break;
                case "Острова":
                    map = new IslandsMap();
                    break;
                case "Скалы":
                    map = new RocksMap();
                    break;
            }
            if (map != null)
            {
                _mapShipCollectionGeneric = new
                MapWithSetShipGeneric<DrawingObjectShip, AbstractMap>(
                pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapShipCollectionGeneric = null;
            }
        }
        private void ButtonAddShip_Click(object sender, EventArgs e)
        {
            if (_mapShipCollectionGeneric == null)
            {
                return;
            }
            FormShip form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                DrawingObjectShip ship = new(form.SelectedShip);
                if (_mapShipCollectionGeneric + ship != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapShipCollectionGeneric.ShowSet();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        
        private void ButtonRemoveShip_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
           MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_mapShipCollectionGeneric - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _mapShipCollectionGeneric.ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (_mapShipCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapShipCollectionGeneric.ShowSet();
            
        }
        
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (_mapShipCollectionGeneric == null)
            {                
                return;
            }
            pictureBox.Image = _mapShipCollectionGeneric.ShowOnMap();
            
        }
       
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_mapShipCollectionGeneric == null)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapShipCollectionGeneric.MoveObject(dir);
        }
    }
}
